from __future__ import absolute_import
import uuid as _uu
from smalluuid.utils import unpack_uuid, pack_uuid


class SmallUUID(_uu.UUID):

    def __init__(self, small=None, hex=None, bytes=None, bytes_le=None,
                       fields=None, int=None, version=None,
                       *args, **kwargs):
        if [hex, bytes, bytes_le, fields, int, small].count(None) != 5:
            int = _uu.uuid4().int

        if small:
            int = unpack_uuid(small).int

        super(SmallUUID, self).__init__(hex=hex, bytes=bytes, bytes_le=bytes_le,
                                        fields=fields, int=int, version=version,
                                        *args, **kwargs)

    @property
    def small(self):
        return pack_uuid(self.hex)

    @property
    def hex_grouped(self):
        return super(SmallUUID, self).__str__()

    def __str__(self):
        return self.small

    def __repr__(self):
        return "{0}('{1}')".format(self.__class__.__name__, str(self))


class TypedSmallUUID(SmallUUID):
    type_bits = 6

    def __init__(self, small=None, hex=None, bytes=None, bytes_le=None,
                       fields=None, int=None, version=None, type=None,
                       *args, **kwargs):
        if [hex, bytes, bytes_le, fields, int, small, type].count(None) != 6:
            raise ValueError(
                'Either a value or the "type" parameter is '
                'required to instantiate a TypedSmallUUID'
            )
        super(TypedSmallUUID, self).__init__(
            small=small, hex=hex, bytes=bytes, bytes_le=bytes_le,
            fields=fields, int=int, version=version,
            *args, **kwargs
        )

        if type is not None:
            self._set_type(type)

    @property
    def type(self):
        return self.int >> 128 - self.type_bits

    def _set_type(self, type_num):
        if not 0 <= type_num < 1 << self.type_bits:
            raise ValueError(
                'Specified type_num of {0} outside range available '
                'to {1} bit types (0 to {2})'.format(
                    type_num, self.type_bits, (1 << self.type_bits) - 1,
                )
            )

        bits = self.int
        id_size = 128 - self.type_bits
        # Zero out the type bits
        bits &= pow(2, id_size) - 1
        # Now add in the type
        bits |= (type_num << id_size)

        if not 0 <= bits < 1 << 128:
            raise ValueError('Addition of type overflowed max value (128-bits max)')

        self.__dict__['int'] = bits


def small_uuid(name=None):
    if name is None:
        uuid = _uu.uuid4()
    elif "http" not in name.lower():
        uuid = _uu.uuid5(_uu.NAMESPACE_DNS, name)
    else:
        uuid = _uu.uuid5(_uu.NAMESPACE_URL, name)
    return SmallUUID(int=uuid.int)



import unittest
import uuid as _uu


class TestUtils(unittest.TestCase):

    def test_pack_unpack(self):
        from smalluuid.utils import unpack_uuid, pack_uuid

        # for x in range(0, 10000):
        #     uuid = _uu.uuid4()
        #     self.assertEqual(uuid, unpack_uuid(pack_uuid(uuid)))

    def test_pack(self):
        from smalluuid.utils import pack_uuid

        uuid = _uu.UUID(int=0, version=4)
        self.assertEqual(pack_uuid(uuid), 'AAAAAAAAQACAAAAAAAAAAA')

        uuid = _uu.UUID(int=int('10' * 64, 2), version=4)
        self.assertEqual(pack_uuid(uuid), 'qqqqqqqqSqqqqqqqqqqqqg')

        uuid = _uu.UUID(int=pow(2, 128) - 1, version=4)
        self.assertEqual(pack_uuid(uuid), '________T_-__________w')

    def test_unpack(self):
        from smalluuid.utils import unpack_uuid

        self.assertEqual(str(unpack_uuid('AAAAAAAAQACAAAAAAAAAAA')),
                         '00000000-0000-4000-8000-000000000000')
        self.assertEqual(str(unpack_uuid('qqqqqqqqSqqqqqqqqqqqqg')),
                         'aaaaaaaa-aaaa-4aaa-aaaa-aaaaaaaaaaaa')
        self.assertEqual(str(unpack_uuid('________T_-__________w')),
                         'ffffffff-ffff-4fff-bfff-ffffffffffff')



from __future__ import absolute_import
from uuid import UUID, uuid4
import base64

 
def pack_uuid(uuid):
    packed_string = base64.urlsafe_b64encode(UUID(str(uuid)).bytes).decode("utf-8")
    packed_string = packed_string.rstrip('=\n').replace('/', '_')
    return packed_string

def unpack_uuid(short_uuid):
    uuid_as_b64 = (short_uuid + '==').replace('_', '/')
    bytes = base64.urlsafe_b64decode(uuid_as_b64.encode("utf-8"))
    return UUID(bytes=bytes)



import unittest
import uuid as _uu


class TestSmallUUIDClass(unittest.TestCase):

    def test_no_input(self):
        from smalluuid.smalluuid import SmallUUID
        uuid = SmallUUID()
        self.assertEqual(uuid.version, 4)

    def test_normal(self):
        from smalluuid.smalluuid import SmallUUID
        self.assertEqual(SmallUUID(int=0).small,
                         'AAAAAAAAAAAAAAAAAAAAAA')

    def test_short(self):
        from smalluuid.smalluuid import SmallUUID
        self.assertEqual(SmallUUID(small='AAAAAAAAAAAAAAAAAAAAAA').small,
                         'AAAAAAAAAAAAAAAAAAAAAA')

    def test_str(self):
        from smalluuid.smalluuid import SmallUUID
        self.assertEqual(str(SmallUUID(small='AAAAAAAAAAAAAAAAAAAAAA')),
                         'AAAAAAAAAAAAAAAAAAAAAA')

    def test_repr(self):
        from smalluuid.smalluuid import SmallUUID
        self.assertEqual(repr(SmallUUID(small='AAAAAAAAAAAAAAAAAAAAAA')),
                         "SmallUUID('AAAAAAAAAAAAAAAAAAAAAA')")

    def test_hex_grouped(self):
        from smalluuid.smalluuid import SmallUUID
        self.assertEqual(SmallUUID(small='AAAAAAAAAAAAAAAAAAAAAA').hex_grouped,
                         "00000000-0000-0000-0000-000000000000")

    def test_with_version(self):
        """Ensure the appropriate version bits get inserted"""
        from smalluuid.smalluuid import SmallUUID
        self.assertEqual(
            SmallUUID(small='AAAAAAAAAAAAAAAAAAAAAA', version=4).hex_grouped,
            "00000000-0000-4000-8000-000000000000"
        )


class TestSmallUUIDFunc(unittest.TestCase):
    def test_uuid4(self):
        from smalluuid.smalluuid import small_uuid
        self.assertEqual(small_uuid().version, 4)

    def test_uuid5_dns(self):
        from smalluuid.smalluuid import small_uuid
        uuid = small_uuid(name="foo.com")
        self.assertEqual(uuid.version, 5)
        self.assertEqual(uuid.hex_grouped, 'cb2b0427-a119-5e0e-b044-484d51ea6a2f')

    def test_uuid5_url(self):
        from smalluuid.smalluuid import small_uuid
        uuid = small_uuid(name="http://foo.com/xyz")
        self.assertEqual(uuid.version, 5)
        self.assertEqual(uuid.hex_grouped, '0742a68a-cfd8-5aa6-9a3d-26dfe936ce64')


class TestTypedSmallUUIDClass(unittest.TestCase):
    def test_get_type(self):
        from smalluuid.smalluuid import TypedSmallUUID
        self.assertEqual(TypedSmallUUID(int=(0b101010 << 122)).type, 0b101010)

    def test_new(self):
        from smalluuid.smalluuid import TypedSmallUUID
        self.assertEqual(TypedSmallUUID(type=63).type, 63)

    def test_overflow(self):
        from smalluuid.smalluuid import TypedSmallUUID
        self.assertRaises(ValueError, TypedSmallUUID, type=64)

    def test_value_or_type_required(self):
        from smalluuid.smalluuid import TypedSmallUUID
        self.assertRaises(ValueError, TypedSmallUUID)



#!/usr/bin/env python

from os.path import exists
from setuptools import setup, find_packages

from smalluuid import __version__

setup(
    name='smalluuid',
    version=__version__,
    # Your name & email here
    author='Adam Charnock',
    author_email='adam@adamcharnock.com',
    packages=find_packages(),
    # Any executable scripts, typically in 'bin'. E.g 'bin/do-something.py'
    scripts=[],
    url='https://github.com/adamcharnock/smalluuid',
    license='BSD',
    description="Provides ShortUUID which extends Python's builtin UUID class",
    # Any requirements here, e.g. "Django >= 1.1.1"
    install_requires=[
        
    ],
)



from __future__ import absolute_import

# It must be possible to import this file with
# none of the package's dependencies installed

__version__ = '0.1.2'

DEFAULT_ALPHABET = tuple("23456789ABCDEFGHJKLMNPQRSTUVWXYZ"
                      "abcdefghijkmnopqrstuvwxyz")

from smalluuid.smalluuid import (
 SmallUUID,
 TypedSmallUUID,
)
